@@ -2,21 +2,51 @@ package ExtUtils::ModuleMaker;
 use strict;
 local $^W = 1;
 use vars qw ($VERSION);
-$VERSION = 0.33;
+$VERSION = 0.34;
 
 use ExtUtils::ModuleMaker::Licenses::Standard;
 use ExtUtils::ModuleMaker::Licenses::Local;
 use File::Path;
+use Carp;
+
+#################### PACKAGE VARIABLES #################### 
 
 #################### PUBLICLY CALLABLE METHODS ####################
 
 #!#!#!#!#
 ##   2 ##
 sub new {
-#    my ( $class, $paramsref ) = @_;
-#    my %parameters = %{$paramsref};
-    my ( $class, %parameters ) = @_;
-    my $self = bless( default_values(), ref($class) || $class );
+    my ( $class, @arglist ) = @_;
+    local $_;
+    croak "Must be hash or balanced list of key-value pairs: $!"
+        if (@arglist % 2);
+    my %parameters = @arglist;
+    my @badkeys;
+    my %keys_permitted = map {$_, 1} qw|
+        NAME
+        ABSTRACT
+        VERSION
+        LICENSE
+        BUILD_SYSTEM
+        AUTHOR
+        EXTRA_MODULES
+        COMPACT
+        VERBOSE
+        INTERACTIVE
+        PERMISSIONS
+        USAGE_MESSAGE
+        NEED_POD
+        NEED_NEW_METHOD
+        CHANGES_IN_POD
+    |;
+    
+    for (keys %parameters) {
+        push(@badkeys, $_) unless $keys_permitted{$_};
+    }
+    croak "Dying due to bad input to constructor (@badkeys): $!"
+        if (@badkeys);
+    my $self = ref($class) ? bless( default_values(), ref($class) )
+                           : bless( default_values(), $class );
     foreach my $param ( keys %parameters ) {
         if ( ref( $parameters{$param} ) eq 'HASH' ) {
             foreach ( keys( %{ $parameters{$param} } ) ) {
@@ -33,6 +63,8 @@ sub new {
     $self->initialize_license();
 
     $self->{MANIFEST} = ['MANIFEST'];
+    $self->verify_values();
+
     return ($self);
 }
 
@@ -41,8 +73,6 @@ sub new {
 sub complete_build {
     my $self = shift;
 
-    $self->verify_values();
-
     $self->create_base_directory();
     $self->check_dir( map { "$self->{Base_Dir}/$_" } qw (lib t scripts) );
 
@@ -69,8 +99,7 @@ sub complete_build {
     }
     else {
         $self->print_file( 'Build.PL', $self->file_text_Buildfile() );
-#        if ( $self->{BUILD_SYSTEM} eq 'Module::Build and proxy Makefile.PL' ) {
-	if ( $self->{BUILD_SYSTEM} eq 'Module::Build and proxy Makefile.PL' 
+        if ( $self->{BUILD_SYSTEM} eq 'Module::Build and proxy Makefile.PL' 
          or  $self->{BUILD_SYSTEM} eq 'Module::Build and Proxy') {
             $self->print_file( 'Makefile.PL',
                 $self->file_text_proxy_makefile() );
@@ -95,14 +124,14 @@ sub complete_build {
 #             are encapsulated within a subroutine rather than creating
 #             a file-scoped lexical. 
 sub default_values {
+    # NAME key is intentionally missing; must cause fatal error
     my %defaults = (
-        NAME     => 'None yet',
         LICENSE  => 'perl',
         VERSION  => 0.01,
         ABSTRACT => 'Module abstract (<= 44 characters) goes here',
         AUTHOR   => {
             NAME         => 'A. U. Thor',
-	    CPANID       => 'AUTHOR',
+            CPANID       => 'AUTHOR',
             ORGANIZATION => 'XYZ Corp.',
             WEBSITE      => 'http://a.galaxy.far.far.away/modules',
             EMAIL        => 'a.u.thor@a.galaxy.far.far.away',
@@ -137,7 +166,7 @@ ENDOFUSAGE
 # Throws    : Will die with a death_message if errors and not interactive.
 # Comments  : 
 sub verify_values {
-    my ($self) = @_;
+    my $self = shift;
     my @errors;
 
     push( @errors, 'NAME is required' )
@@ -145,13 +174,12 @@ sub verify_values {
     push( @errors, 'ABSTRACTs are limited to 44 characters' )
       if ( length( $self->{ABSTRACT} ) > 44 );
     push( @errors, 'CPAN IDs are 3-9 characters' )
-      if ( ( exists( $self->{AUTHOR}{CPANID} ) )
-        && ( $self->{AUTHOR}{CPANID} !~ m/^\w{3,9}$/ ) );
+      if ( $self->{AUTHOR}{CPANID} !~ m/^\w{3,9}$/ );
     push( @errors, 'EMAIL addresses need to have an at sign' )
       if ( $self->{AUTHOR}{EMAIL} !~ m/.*\@.*/ );
     push( @errors, 'WEBSITEs should start with an "http:" or "https:"' )
       if ( $self->{AUTHOR}{WEBSITE} !~ m/https?:\/\/.*/ );
-    push( @errors, 'LICENSE is not recognized"' )
+    push( @errors, 'LICENSE is not recognized' )
       unless ( Verify_Local_License( $self->{LICENSE} )
         || Verify_Standard_License( $self->{LICENSE} ) );
 
@@ -209,7 +237,7 @@ sub set_dates {
 #!#!#!#!#
 ##  11 ##
 sub set_author_data {
-    my ($self) = @_;
+    my $self = shift;
 
     my $p_author = $self->{AUTHOR};
     $p_author->{COMPOSITE} = (
@@ -253,7 +281,7 @@ sub create_pm_basics {
 #!#!#!#!#
 ##  15 ##
 sub initialize_license {
-    my ($self) = @_;
+    my $self = shift;
 
     $self->{LICENSE} = lc( $self->{LICENSE} );
 
@@ -327,10 +355,15 @@ sub check_dir {
 ##  20 ##
 sub death_message {
     my $self = shift;
+    my @errors = @_;
 
-    die( join "\n", @_, '', $self->{USAGE_MESSAGE} )
+    croak( join "\n", @errors, '', $self->{USAGE_MESSAGE} )
       unless $self->{INTERACTIVE};
-    print( join "\n", 'Oops, there are the following errors:', @_, '', '' );
+    my %err = map {$_, 1} @errors;
+    delete $err{'NAME is required'} if $err{'NAME is required'};
+    @errors = keys %err;
+    print( join "\n", 'Oops, there are the following errors:', @errors, '', '' )
+        if @errors;
 }
 
 #!#!#!#!#
@@ -566,7 +599,7 @@ EOFBLOCK
 # Throws    : n/a
 # Comments  : This method is a likely candidate for alteration in a subclass
 sub file_text_README {
-    my ($self) = @_;
+    my $self = shift;
 
     my $build_instructions;
     if ( $self->{BUILD_SYSTEM} eq 'ExtUtils::MakeMaker' ) {
@@ -651,7 +684,7 @@ EOF
 # Throws    : n/a
 # Comments  : This method is a likely candidate for alteration in a subclass
 sub file_text_ToDo {
-    my ($self) = @_;
+    my $self = shift;
 
     my $page = <<EOF;
 TODO list for Perl module $self->{NAME}
@@ -673,22 +706,28 @@ EOF
 # Throws    : n/a
 # Comments  : This method is a likely candidate for alteration in a subclass
 sub file_text_Makefile {
-    my ($self) = @_;
-    my $page = <<EOF;
+    my $self = shift;
+    my $page = sprintf q~
+
 use ExtUtils::MakeMaker;
 # See lib/ExtUtils/MakeMaker.pm for details of how to influence
 # the contents of the Makefile that is written.
 WriteMakefile(
-    NAME         => '$self->{NAME}',
-    VERSION_FROM => '$self->{FILE}', # finds \$VERSION
-    AUTHOR       => '$self->{AUTHOR}{NAME} ($self->{AUTHOR}{EMAIL})',
-    ABSTRACT     => '$self->{ABSTRACT}',
+    NAME         => '%s',
+    VERSION_FROM => '%s', # finds \$VERSION
+    AUTHOR       => '%s (%s)',
+    ABSTRACT     => '%s',
     PREREQ_PM    => {
                      'Test::Simple' => 0.44,
                     },
 );
-EOF
-    return ($page);
+~,  map { my $s = $_; $s =~ s{'}{\\'}g; $s; }
+    $self->{NAME},
+    $self->{FILE},
+    $self->{AUTHOR}{NAME},
+    $self->{AUTHOR}{EMAIL},
+    $self->{ABSTRACT};
+    return $page;
 }
 
 #!#!#!#!#
@@ -700,7 +739,7 @@ EOF
 # Throws    : n/a
 # Comments  : This method is a likely candidate for alteration in a subclass
 sub file_text_Buildfile {
-    my ($self) = @_;
+    my $self = shift;
 
     # As of 0.15, Module::Build only allows a few licenses
     my $license_line = 1 if $self->{LICENSE} =~ /^(?:perl|gpl|artistic)$/;
@@ -738,7 +777,7 @@ EOF
 # Throws    : n/a
 # Comments  : This method is a likely candidate for alteration in a subclass
 sub file_text_proxy_makefile {
-    my ($self) = @_;
+    my $self = shift;
 
     # This comes directly from the docs for Module::Build::Compat
     my $page = <<'EOF';
@@ -904,12 +943,19 @@ is probably best held in a hash.   Keys which may be specified are:
 
 The I<only> required feature.  This is the name of the primary module 
 (with 'C<::>' separators if needed).  Will also support the older style 
-separator ''C<'>'' like the module F<D'Oh>.  Current default is 'None yet'. 
+separator ''C<'>'' like the module F<D'Oh>, though this feature is
+deprecated and is subject to removal in a future version.  In any event,
+if the module name contains an apostrophe, then the value corresponding
+to key <NAME> in the list passed must be double-quoted; otherwise
+F<Makefile.PL> gets messed up.  There is no current default for NAME.
 
 =item * ABSTRACT
 
 A short (44-character maximum) description of the module.  CPAN likes 
-to use this feature to describe the module.
+to use this feature to describe the module.  If the abstract contains an
+apostrophe (C<'>), then the value corresponding to key C<ABSTRACT> in
+the list passed to the constructor must be double-quoted; otherwise
+F<Makefile.PL> gets messed up.
 
 =item * VERSION
 
@@ -944,7 +990,7 @@ The first generates a basic Makefile.PL file for your module.
 
 The second creates a Build.PL file.
 
-=item * C<Module::Build and Proxy>
+=item * C<'Module::Build and Proxy'>
 
 The third creates a Build.PL along with a proxy Makefile.PL
 script that attempts to install Module::Build if necessary, and then
@@ -971,11 +1017,16 @@ necessary places in the files.
 
 =item * NAME
 
-Name of the author.
+Name of the author.  If the author's name contains an apostrophe (C<'>), 
+then the corresponding value in the list passed to the constructor must 
+be double-quoted; otherwise F<Makefile.PL> gets messed up.
 
 =item * EMAIL
 
-Email address of the author.
+Email address of the author.  If the author's e-mail address contains 
+an apostrophe (C<'>), then the corresponding value in the list passed 
+to the constructor must be double-quoted; otherwise
+F<Makefile.PL> gets messed up.
 
 =item * CPANID
 
@@ -1052,11 +1103,6 @@ passed to C<ExtUtils::ModuleMaker::new>.  Returns a
 true value if all specified files are created -- but this says nothing
 about whether those files have been created with the correct content.
 
-=head1 SUPPORT
-
-Send email to jkeenan [at] cpan [dot] org.  Please include 'modulemaker'
-in the subject line.
-
 =head1 AUTHOR
 
 ExtUtils::ModuleMaker was originally written in 2001-02 by R. Geoffrey Avery
@@ -1064,6 +1110,17 @@ ExtUtils::ModuleMaker was originally written in 2001-02 by R. Geoffrey Avery
 2005) it has been maintained by James E. Keenan (jkeenan [at] cpan [dot]
 org).
 
+=head1 SUPPORT
+
+Send email to jkeenan [at] cpan [dot] org.  Please include 'modulemaker'
+in the subject line.
+
+=head1 ACKNOWLEDGMENTS
+
+Thanks to Geoff Avery for inventing and popularizing
+ExtUtils::Modulemaker.  Thanks for bug reports and fixes to David A
+Golden and an anonymous guest on rt.cpan.org.
+
 =head1 COPYRIGHT
 
 Copyright (c) 2001-2002 R. Geoffrey Avery. All rights reserved.
@@ -1078,3 +1135,4 @@ LICENSE file included with this module.
 F<modulemaker>, F<perlnewmod>, F<h2xs>, F<ExtUtils::MakeMaker>.
 
 =cut
+
